/***********************************************************************
*  (c)       : ABB Automation Products GmbH                            *
************************************************************************
*  System    : PDM 6.0+SP1+HF1
*-----------------------------------------------------------------------
*  Compiler  : PDM 6.0
*-----------------------------------------------------------------------
*  Date      :
*-----------------------------------------------------------------------
*  Revision  : 0.1.1 (DD_REVISION 1)
*-----------------------------------------------------------------------
*  Author    : Pradipta Kishore Bala (INCRC,INDIA)
******************+*****************************************************
*  Device:		ABB TTR200
*  Communication:	HART
***********************************************************************/


/*****************************************************************************
 * The manufacturer line
 *****************************************************************************
 */
MANUFACTURER ABB, DEVICE_TYPE _TTX200, DEVICE_REVISION 2, DD_REVISION 1

/*****************************************************************************
 * Include Files
 *****************************************************************************
 */

#include "macros.h"
#include "methods.h"
#include "TTX200.h"
#include "snsrrule.dd"
#include "snsr1_1.dd"
#include "tempcmd.dd"
#include "diag.dd"
#include "menus.dd"
#include "Elcetrical1_1.dd"
#include "ElcetronicsTemp.dd"
#include "methods.dd"


/*****************************************************************************
 * Definitions (#define, typedef, enum)
 *****************************************************************************
 */
#ifdef	_PDM_
#define	HAS_SOFTWARE_VERSION_STRING	TRUE
#define	HAS_HARDWARE_VERSION_STRING	TRUE
#endif

/*****************************************************************************
 * Common Tables
 */
IMPORT STANDARD _TABLES, DEVICE_REVISION 15, DD_REVISION 1
{
    ALL_TABLES;
}

/*****************************************************************************
 * Universal Commmands
 */

IMPORT STANDARD _UNIVERSAL, DEVICE_REVISION 5, DD_REVISION 2
{
    EVERYTHING;

    REDEFINITIONS
    {

    	COMMAND read_pv_sensor_info
    	{
        	REDEFINE TRANSACTION
        	{
            		REQUEST {}
            		REPLY
            		{
            		    response_code, device_status,
            		    deviceVariables[0].SENSOR_SERIAL_NUMBER,
            		    deviceVariables[0].DIGITAL_UNITS,
            		    deviceVariables[0].UPPER_SENSOR_LIMIT,
            		    deviceVariables[0].LOWER_SENSOR_LIMIT,
            		    deviceVariables[0].MINIMUM_SPAN
            		}
        	}
	}

	COMMAND read_pv_output_info
	{
		REDEFINE TRANSACTION
		    {
		        REQUEST {}
		        REPLY
		        {
		            response_code, device_status,
		            PV.DAQ.ALARM_CODE,
		            PV.RANGING.TRANSFER_FUNCTION,
		            pv_range_units,
		            pv_upperRange_value,//PV.RANGING.UPPER_RANGE_VALUE,
		            pv_lowerRange_value,//PV.RANGING.LOWER_RANGE_VALUE,
		            deviceVariables[0].DAMPING_VALUE,
		            write_protect,
		            private_label_distributor
		        }
		    }
	}

	COMMAND read_pv_current_and_percent_range
	{
		REDEFINE TRANSACTION
		    {
		        REQUEST {}
		        REPLY
		        {
		            response_code, device_status,
		            pv_loopCurrent,
		            pv_percentRange
		        }
		    }

	}

	COMMAND read_dynamic_variables_and_pv_current
	{
		REDEFINE TRANSACTION
		    {
		        REQUEST {}
		        REPLY
		        {
		            response_code, device_status,
		            pv_loopCurrent,
		            deviceVariables[0].DIGITAL_UNITS, deviceVariables[0].DIGITAL_VALUE,
		            SV.DEVICE_VARIABLE.DIGITAL_UNITS, SV.DEVICE_VARIABLE.DIGITAL_VALUE,
		            TV.DEVICE_VARIABLE.DIGITAL_UNITS, TV.DEVICE_VARIABLE.DIGITAL_VALUE
		        }
		    }

	}

   	VARIABLE manufacturer_id
        {
            REDEFINE TYPE ENUMERATED
            {
                { 26,  "ABB" }
            }
        }

        VARIABLE device_type
        {
            REDEFINE TYPE ENUMERATED
            {
                { 13,  "|en|TTX200 series|de|TTX200 Serie" }
            }
        }

        VARIABLE private_label_distributor
        {
            REDEFINE TYPE ENUMERATED
            {
                { 26, "ABB" }
            }
        }
        VARIABLE polling_address
        {
            REDEFINE HANDLING WRITE;
        }
    }
}

IMPORT STANDARD _PV, DEVICE_REVISION 1, DD_REVISION 1
{
    EVERYTHING;

    REDEFINITIONS
    {
        DELETE VARIABLE loop_flags;
        ARRAY OF COLLECTION dynamic_variables
	{
	  ELEMENTS
	    {
	     DELETE   QUATERNARY;
	    }
	}

	COLLECTION OF COLLECTION secondary_variable
			{
				MEMBERS
				{
					REDEFINE DEVICE_VARIABLE, deviceVariables[8];
				}
			}
	COLLECTION OF COLLECTION tertiary_variable
			{
				MEMBERS
				{
					REDEFINE DEVICE_VARIABLE, deviceVariables[5];
				}
			}
    	DELETE COLLECTION quaternary_variable;

        COLLECTION OF VARIABLE analog_io
        {
            MEMBERS
            {
                DELETE ANALOG_CHANNEL_FLAGS;
            }
        }

        COLLECTION OF VARIABLE scaling
        {
            MEMBERS
            {

                ADD RANGE_UNITS, deviceVariables[0].RANGE_UNITS, [range_units], [range_units_help];
            }
        }

        REDEFINE UNIT scaling_units_relation
        {
            	deviceVariables[0].DIGITAL_UNITS:
               upperRange_value,
               lowerRange_value
        }
    }
}

/*****************************************************************************
 * Common Practice Commands
 */

/******************************************************************************
*                                  Common Practice Commands
******************************************************************************/
IMPORT STANDARD _COMMON_PRACTICE, DEVICE_REVISION 7, DD_REVISION 3
{
	VARIABLE device_variable_code;
	COMMAND write_pv_damping_value;                 /* command 34 */
//	COMMAND write_pv_range_values;                  /* command 35 */
	COMMAND set_pv_upper_range_value;		/* command 36 */
	COMMAND set_pv_lower_range_value;		/* command 37 */
	COMMAND reset_configuration_change_flag;        /* command 38 */
	COMMAND enter_exit_fixed_pv_current_mode;       /* command 40 */
	COMMAND perform_self_test;			/* command 41 */
	COMMAND perform_device_reset;			/* command 42 */
	COMMAND write_pv_units;                         /* command 44 */
	COMMAND trim_pv_current_dac_zero;               /* command 45 */
	COMMAND trim_pv_current_dac_gain;               /* command 46 */
	COMMAND read_additional_device_status;          /* command 48 */
	COMMAND write_pv_sensor_serial_number;		/* command 49 */
	COMMAND write_number_of_response_preambles;	/* command 59 */

	METHOD warning_message;
	METHOD return_to_normal;

    REDEFINITIONS
	{

	COMMAND write_pv_damping_value
		{

		    REDEFINE TRANSACTION
		    {
		        REQUEST
		        {
		            damping_value
		        }
		        REPLY
		        {
		            response_code, device_status, damping_value
		        }
		    }
		}
	
	COMMAND	enter_exit_fixed_pv_current_mode
	{
		REDEFINE TRANSACTION
		{
			REQUEST
			{
				pv_loopCurrent
			}
			REPLY
			{
				response_code, device_status, pv_loopCurrent
			}
		}
	}
	
	COMMAND	trim_pv_current_dac_zero
	{
		REDEFINE TRANSACTION
		{
			REQUEST
			{
				pv_loopCurrent
			}
			REPLY
			{
				response_code, device_status, pv_loopCurrent
			}
		}
	}
	
	COMMAND	trim_pv_current_dac_gain
	{
		REDEFINE TRANSACTION
		{
			REQUEST
			{
				pv_loopCurrent
			}
			REPLY
			{
				response_code, device_status, pv_loopCurrent
			}
		}
	}
	
	COMMAND	write_pv_units
	{
		REDEFINE TRANSACTION
		{
			REQUEST
			{
				sensor1_digital_units
			}
			REPLY
			{
				response_code, device_status,sensor1_digital_units
			}
		}
	}

	COMMAND read_additional_device_status
        {
            REDEFINE TRANSACTION
            {
                REQUEST
                    {}

                REPLY
                {
                    response_code, device_status,

                    DiagApplication_A,
                    DiagApplication_B
                    DiagSensor1_A,
                    DiagSensor1_B,
                    DiagReserved_0,
                    DiagReserved_1,
                    DiagReserved_5,
                    DiagReserved_2,
                    DiagReserved_3,
                    DiagElectronics_A,
                    DiagElectronics_B,
                    DiagCommunication_A,
                    DiagCommunication_B,
                    NAMUR_state
                }
            }
        }
    }
}

METHOD update_pv_range_unit
{
    LABEL "Update PV range unit";
    DEFINITION
    {
        assign_var(PV.DEVICE_VARIABLE.RANGE_UNITS, PV.DEVICE_VARIABLE.DIGITAL_UNITS);
    }
}

VARIABLE software_version_major
{
    LABEL "SW version major";
    CLASS DEVICE;
    HANDLING READ;
    TYPE UNSIGNED_INTEGER;
}

VARIABLE software_version_minor
{
    LABEL "SW version minor";
    CLASS DEVICE;
    HANDLING READ;
    TYPE UNSIGNED_INTEGER;
}

VARIABLE software_version_revision
{
    LABEL "SW version revision";
    CLASS DEVICE;
    HANDLING READ;
    TYPE UNSIGNED_INTEGER;
}

VARIABLE hardware_version_major
{
    LABEL "HW version major";
    CLASS DEVICE;
    HANDLING READ;
    TYPE UNSIGNED_INTEGER;
}

VARIABLE hardware_version_minor
{
    LABEL "HW version minor";
    CLASS DEVICE;
    HANDLING READ;
    TYPE UNSIGNED_INTEGER;
}

#ifdef HAS_SOFTWARE_VERSION_STRING
VARIABLE software_version
{
    LABEL SOFTWARE_VERSION;
    CLASS LOCAL;
    TYPE ASCII(10);
    VALIDITY HAS_SOFTWARE_VERSION_STRING;
    REFRESH_ACTIONS
    {
        update_software_version
    }
}

METHOD update_software_version
{
   LABEL   "Update SW version info";
   DEFINITION
   {
       software_version =
           itoa(software_version_major) + "."
           + itoa(software_version_minor) + "."
           + itoa(software_version_revision);
   }
}
REFRESH version {
    software_version_major, software_version_minor, software_version_revision:
        software_version
}

#endif

#ifdef HAS_HARDWARE_VERSION_STRING
VARIABLE hardware_version
{
    LABEL HARDWARE_VERSION;
    CLASS LOCAL;
    TYPE ASCII(10);
    VALIDITY HAS_HARDWARE_VERSION_STRING;
    REFRESH_ACTIONS
    {
        update_hardware_version
    }
}

METHOD update_hardware_version
{
   LABEL   "Update HW version info";
   DEFINITION
   {
       hardware_version =
           itoa(hardware_version_major) + "."
           + itoa(hardware_version_minor);
   }
}

#endif

COMMAND READ_REVISIONS_AND_WRITE_PROTECT_STATE
{
    NUMBER 171;
    OPERATION READ;
    TRANSACTION
    {
        REQUEST
        {
        }
        REPLY
        {
            response_code, device_status,
            software_version_major,
            software_version_minor,
            software_version_revision,
            write_protection_state,
            hardware_version_major,
            hardware_version_minor
        }
    }
    RESPONSE_CODES
    {
        0,  SUCCESS,          [no_command_specific_errors];
        2,  DATA_ENTRY_ERROR, [invalid_selection];
        3,  DATA_ENTRY_ERROR, [passed_parameter_too_large];
        4,  DATA_ENTRY_ERROR, [passed_parameter_too_small];
        5,  DATA_ENTRY_ERROR, [too_few_data_bytes_recieved];
        7,  MODE_ERROR,       [in_write_protect_mode];
        16, MODE_ERROR,       [access_restricted];
        32, MODE_ERROR,       [busy];
    }
}

VARIABLE sensor_serial_number
{
    HELP [sensor_serial_number_help];
    LABEL "Sensor serial number";
    TYPE UNSIGNED_INTEGER (3)
    {
        DISPLAY_FORMAT "1u"; /* minimum field width */
        EDIT_FORMAT "8u";
        MAX_VALUE 16777215;
    }
    CLASS DEVICE & SERVICE;
    HANDLING READ & WRITE;
}

VARIABLE damping_value
{
    HELP [seconds_damping_value_help];
    LABEL "|en|Damping|de|Dmpfung";
    TYPE FLOAT
    {
        DISPLAY_FORMAT "3.1f";
        EDIT_FORMAT "4.1f";
        MIN_VALUE 0.0;
        MAX_VALUE 32.0;
    }
    CONSTANT_UNIT "s";
    CLASS INPUT & SERVICE;
    HANDLING READ & WRITE;
}

VARIABLE range_units
{
    HELP [digital_units_help];
    LABEL "Range unit";
    TYPE ENUMERATED
    {
        ALL_TTX_UNITS
    }
    CLASS INPUT & SERVICE;
    HANDLING READ & WRITE;
}


/****************************************************************************/
/*				COLLECTION				*/
/****************************************************************************/



ARRAY OF COLLECTION deviceVariables
{
    ELEMENTS
    {
        0, sensor1_measurement,          "Sensor";
        5, electrical1_measurement,      "|en|Electric input |de|Elektronik Eingang";
        8, electronics_temperature,      "|en|Electronics temperature|de|Elektroniktemperatur";
    }
}


/****************************************************************************/



MENU download_variables
{
    LABEL "|en|upload list|de|uploade liste";

    ITEMS
    {
        tag,
        descriptor,
        date,
        message,

        manufacturer_id,
        device_type,
        device_id,
        software_revision,
        hardware_revision,

        software_version_major,
        software_version_minor,
        software_version_revision,

        hardware_version_major,
        hardware_version_minor,

        write_protect,

        PV.DAQ.ALARM_CODE,
        sensor_serial_number,
        PV.RANGING.TRANSFER_FUNCTION,
        PV.DEVICE_VARIABLE.DAMPING_VALUE,

        PV.DEVICE_VARIABLE.DIGITAL_UNITS,
        pv_range_units,
        pv_upperRange_value,//PV.RANGING.UPPER_RANGE_VALUE,
        pv_lowerRange_value,//PV.RANGING.LOWER_RANGE_VALUE,

        sensor1_probe_type,
        sensor1_wire_number,
        sensor1_probe_connection,
        sensor1_CJC_type,
        sensor1_CJC_temperature_unit,
        sensor1_CJC_temperature,
        sensor1_resistance_two_wire,
        sensor1_bias,

	sensor1_upper_sensor_limit,
	sensor1_lower_sensor_limit,
        sensor1_minimum_span
        sensor1_trim_low,

        AO_REACTION, AO_UNDERRANGE_VALUE, AO_OVERRANGE_VALUE

    }
}

MENU upload_variables
{
    LABEL "|en|download list|de|uploade liste";

    ITEMS
    {
        tag,
        descriptor,
        date,
        message,


        PV.DAQ.ALARM_CODE,
        PV.RANGING.TRANSFER_FUNCTION,
        PV.DEVICE_VARIABLE.DAMPING_VALUE,

        PV.DEVICE_VARIABLE.DIGITAL_UNITS,
        pv_range_units,
        pv_upperRange_value,//PV.RANGING.UPPER_RANGE_VALUE,
        pv_lowerRange_value,//PV.RANGING.LOWER_RANGE_VALUE,

        sensor1_probe_type,
        sensor1_wire_number,
        sensor1_probe_connection,
        sensor1_CJC_type,
        sensor1_CJC_temperature_unit,
        sensor1_CJC_temperature,
        sensor1_resistance_two_wire,
        sensor1_bias,
        AO_REACTION, AO_UNDERRANGE_VALUE, AO_OVERRANGE_VALUE

    }
}

MENU root_menu
{
    LABEL MENU_ROOT_MENU;
    STYLE MENU;
    ITEMS
    {
        identification,
        configuration,
        parameterization,
        process_variables_root_menu,
        diagnostic_root_menu,
        simulation,
        maintenance,
        extra
    }
}

MENU offline_root_menu
{
    LABEL "Offline";
    STYLE MENU;
    ITEMS
    {
        identification,
        configuration,
        parameterization
    }
}

MENU device_root_menu
{
    LABEL [bus_devices];
    STYLE MENU;
    ITEMS
    {
        identification,
        configuration,
        parameterization,
        simulation,
        maintenance,
        extra
    }
}

MENU cfg_sensor1
{
    LABEL "Sensor";
    STYLE GROUP;
    ITEMS
    {
        sensor1_probe_type (DISPLAY_VALUE),
        sensor1_wire_number (DISPLAY_VALUE),
        sensor1_resistance_two_wire (DISPLAY_VALUE),
        sensor1_CJC_type (DISPLAY_VALUE),
        sensor1_CJC_temperature_unit (DISPLAY_VALUE),
        sensor1_CJC_temperature (DISPLAY_VALUE),
        sensor1_upper_sensor_limit (DISPLAY_VALUE),
        sensor1_lower_sensor_limit (DISPLAY_VALUE),
        sensor1_minimum_span (DISPLAY_VALUE)

    }
}

MENU cfg_sensor1_review
{
    LABEL "Sensor";
    STYLE GROUP;
    ITEMS
    {
        sensor1_probe_type (DISPLAY_VALUE, READ_ONLY),
        sensor1_wire_number  (DISPLAY_VALUE, READ_ONLY),
        sensor1_resistance_two_wire  (DISPLAY_VALUE, READ_ONLY),
        sensor1_CJC_type  (DISPLAY_VALUE, READ_ONLY),
//        sensor1_CJC_temperature_unit(DISPLAY_VALUE, READ_ONLY),
        sensor1_CJC_temperature (DISPLAY_VALUE, READ_ONLY),
        sensor1_upper_sensor_limit  (DISPLAY_VALUE, READ_ONLY),
        sensor1_lower_sensor_limit (DISPLAY_VALUE, READ_ONLY),
        sensor1_minimum_span  (DISPLAY_VALUE, READ_ONLY)
    }
}

MENU configuration
{
    LABEL MENU_CONFIGURE;
    STYLE WINDOW;
    ITEMS
    {
        cfg_sensor1
    }
}

VARIABLE AO_OVERRANGE_VALUE
{
    LABEL "|en|Overrange value|de|Obergrenzwert";
    CLASS CORRECTION;
    HANDLING READ & WRITE;
    DEFAULT_VALUE 22;
    TYPE FLOAT
    {
        MIN_VALUE	20.0;
        MAX_VALUE	23.0;
    }
    CONSTANT_UNIT "mA";
    VALIDITY	IF (AO_REACTION==0){TRUE;}ELSE{FALSE;}
}

VARIABLE AO_UNDERRANGE_VALUE
{
    LABEL "|en|Underrange value|de|Untergrenzwert";
    CLASS CORRECTION;
    HANDLING READ & WRITE;
    DEFAULT_VALUE 3.0;
    TYPE FLOAT
    {
    MAX_VALUE	4.0;
    MIN_VALUE	3.6;

    }
    CONSTANT_UNIT "mA";
    VALIDITY IF (AO_REACTION==1){TRUE;}ELSE{FALSE;}
}

VARIABLE AO_REACTION
{
    LABEL "|en|Reaction on errors|de|Reaktion bei Fehler";
    CLASS CORRECTION;
    HANDLING READ & WRITE;
    DEFAULT_VALUE 0;
    TYPE ENUMERATED
    {
        { 0, "|en|Overrange|de|bersteuern" }
        { 1, "|en|Underrange|de|Untersteuern" }
    }
}

COMMAND read_current_out_alarm_configuration
{
    NUMBER 174;
    OPERATION READ;
    TRANSACTION
    {
        REQUEST
        {
        }
        REPLY
        {
            response_code, device_status,
            AO_OVERRANGE_VALUE,
            AO_UNDERRANGE_VALUE,
            AO_REACTION
        }
    }
    RESPONSE_CODES
    {
        0,  SUCCESS,          [no_command_specific_errors];
        2,  DATA_ENTRY_ERROR, [invalid_selection];
        3,  DATA_ENTRY_ERROR, [passed_parameter_too_large];
        4,  DATA_ENTRY_ERROR, [passed_parameter_too_small];
        5,  DATA_ENTRY_ERROR, [too_few_data_bytes_recieved];
        7,  MODE_ERROR,       [in_write_protect_mode];
        16, MODE_ERROR,       [access_restricted];
        32, MODE_ERROR,       [busy];
    }
}

COMMAND write_current_out_alarm_configuration
{
    NUMBER 175;
    OPERATION WRITE;
    TRANSACTION
    {
        REQUEST
        {
            AO_OVERRANGE_VALUE,
            AO_UNDERRANGE_VALUE,
            AO_REACTION
        }
        REPLY
        {
            response_code, device_status,
            AO_OVERRANGE_VALUE,
            AO_UNDERRANGE_VALUE,
            AO_REACTION
        }
    }
    RESPONSE_CODES
    {
        0,  SUCCESS,          [no_command_specific_errors];
        2,  DATA_ENTRY_ERROR, [invalid_selection];
        3,  DATA_ENTRY_ERROR, [passed_parameter_too_large];
        4,  DATA_ENTRY_ERROR, [passed_parameter_too_small];
        5,  DATA_ENTRY_ERROR, [too_few_data_bytes_recieved];
        7,  MODE_ERROR,       [in_write_protect_mode];
        16, MODE_ERROR,       [access_restricted];
        32, MODE_ERROR,       [busy];
    }
}
MENU current_output
{
    LABEL "|en|Current output|de|Ausgangsstrom";
    STYLE GROUP;
    ITEMS
    {
        sensor1_digital_units,//PV.DEVICE_VARIABLE.DIGITAL_UNITS (DISPLAY_VALUE),
//        pv_range_units,//PV.RANGING.RANGE_UNITS (DISPLAY_VALUE)
        pv_lowerRange_value,//PV.RANGING.LOWER_RANGE_VALUE (DISPLAY_VALUE),
        pv_upperRange_value,//PV.RANGING.UPPER_RANGE_VALUE (DISPLAY_VALUE),
        damping_value,//PV.DEVICE_VARIABLE.DAMPING_VALUE (DISPLAY_VALUE),
        AO_REACTION, AO_UNDERRANGE_VALUE, AO_OVERRANGE_VALUE
    }
}

MENU parameterization
{
    LABEL MENU_PARAMETERIZE;
    STYLE WINDOW;
    ITEMS
    {
        cfg_sensor1_review,
        COLUMNBREAK,
        current_output
    }
}

MENU process_variables_root_menu
{
    LABEL MENU_OBSERVE;
    STYLE WINDOW;
    ITEMS
    {
	observetion
    }
}

MENU	observetion
{
	LABEL	MENU_OBSERVE;
	STYLE	GROUP;
	ITEMS
	{
		pv_percentRange(DISPLAY_VALUE),
		pv_loopCurrent (DISPLAY_VALUE),
		PV.DEVICE_VARIABLE.DIGITAL_VALUE (DISPLAY_VALUE),
		SV.DEVICE_VARIABLE.DIGITAL_VALUE (DISPLAY_VALUE),
		TV.DEVICE_VARIABLE.DIGITAL_VALUE (DISPLAY_VALUE)
	}
}

/****************************************************************************/
/******************************	TRIM	*************************************/
/****************************************************************************/

VARIABLE sensor1_trim_unit
{
    LABEL "|en|sensor trim low|de|Unterer Abgleichpunkt";
    CLASS CORRECTION;
    HANDLING READ & WRITE;
    TYPE ENUMERATED(1)
    {
        ALL_TTX_UNITS
    }
}

VARIABLE sensor1_trim_value
{
    LABEL "|en|sensor trim Value|de|Sensor Abgleichswert";
    CLASS CORRECTION;
    HANDLING WRITE;
    TYPE FLOAT;
}

VARIABLE sensor1_trim_low
{
    LABEL "|en|sensor trim low|de|Unterer Abgleichwert";
    CLASS CORRECTION;
    HANDLING READ & WRITE;
    TYPE FLOAT
    {
	DISPLAY_FORMAT "3.2f";
	EDIT_FORMAT "4.1f";
//	MIN_VALUE 0.00;
//	MAX_VALUE 100.99;
    }
}

VARIABLE sensor1_trim_high
{
    LABEL "|en|sensor trim high|de|Oberer Abgleichwert";
    CLASS CORRECTION;
    HANDLING READ;
    TYPE FLOAT
	{
	    	DISPLAY_FORMAT "3.1f";
	    	EDIT_FORMAT "4.1f";
//	    	MIN_VALUE 0.0;
//	    	MAX_VALUE 100.99;
	}
}

VARIABLE sensor1_trim_support
{
    LABEL "|en|Sensor trim support|de|Sensor Abgleich";
    HELP [trim_support_help];
    HANDLING WRITE;
    TYPE ENUMERATED (1)
    {
        {1,	"|en|1-point trim|de|1-Punkt Abgleich"},
        {2,	"|en|2-point trim|de|2-Punkt Abgleich"},
        {3,	"|en|1-point and 2-point trim|de|1-Punkt und 2-Punkt Abgleich"}
    }
}




MENU maintenance_sensor_trim
{
    LABEL "Sensor Trim";
    STYLE GROUP;
    ITEMS 
    {  
        trim_sensor1_one_point,
        COLUMNBREAK,        
        trim_sensor1_reset
    }
}

MENU maintenance_sensor1_co_trim
{
    LABEL "Current Output Trim";
    STYLE GROUP;
    ITEMS 
    {  
	transmitter_dac_trim
    }
}

MENU maintenance
{
    LABEL MENU_CALIBRATE;
    STYLE WINDOW;
    ITEMS
    {
        maintenance_sensor_trim,
        maintenance_sensor1_co_trim    
    }
}

COMMAND	write_device_variable_trim_point
{
	NUMBER	226;
	OPERATION WRITE;
	TRANSACTION
	{
		REQUEST
		{
			device_variable_code,
			sensor1_trim_support,
			sensor1_trim_unit,
			sensor1_trim_low
		}
		REPLY
		{
			response_code, device_status,
			device_variable_code,
			sensor1_trim_support,
			sensor1_trim_unit,
			sensor1_trim_low
		}

	}
    	RESPONSE_CODES
    	{
    	    0,  SUCCESS,          [no_command_specific_errors];
    	    2,  DATA_ENTRY_ERROR, [invalid_selection];
    	    3,  DATA_ENTRY_ERROR, [passed_parameter_too_large];
    	    4,  DATA_ENTRY_ERROR, [passed_parameter_too_small];
    	    5,  DATA_ENTRY_ERROR, [too_few_data_bytes_recieved];
    	    6,  MISC_ERROR,         [xmtr_specific_command_error];
    	    7,  MODE_ERROR,       [in_write_protect_mode];
    	    9,  DATA_ENTRY_ERROR,   [lower_range_value_too_high];
            10, DATA_ENTRY_ERROR,   [lower_range_value_too_low];
            11, DATA_ENTRY_ERROR,   [upper_range_value_too_high];
            13, DATA_ENTRY_ERROR,   [URV_and_LRV_out_of_limits];
            14, DATA_ENTRY_WARNING, [span_too_small];
            16, MODE_ERROR,         [access_restricted];
            32, MODE_ERROR,       [busy];

    	}
}

COMMAND	reset_device_variable_trim_point
{
	NUMBER	227;
	OPERATION COMMAND;
	TRANSACTION
	{
		REQUEST
		{
			device_variable_code
		}
		REPLY
		{
			response_code, device_status,
			device_variable_code
		}

	}
    	RESPONSE_CODES
    	{
    	    0,  SUCCESS,          [no_command_specific_errors];
    	    2,  DATA_ENTRY_ERROR, [invalid_selection];
    	    5,  DATA_ENTRY_ERROR, [too_few_data_bytes_recieved];
    	    6,  MISC_ERROR,       [xmtr_specific_command_error];
    	    7,  MODE_ERROR,       [in_write_protect_mode];
            16, MODE_ERROR,       [access_restricted];
            32, MODE_ERROR,       [busy];

    	}
}

COMMAND	read_device_variable_trim_points
{
	NUMBER	224;
	OPERATION READ;
	TRANSACTION
	{
		REQUEST
		{
			device_variable_code
		}
		REPLY
		{
			response_code, device_status,
			device_variable_code,
			sensor1_trim_unit,
			sensor1_trim_low,
			sensor1_trim_high
		}

	}
    	RESPONSE_CODES
    	{
    	    0,  SUCCESS,          [no_command_specific_errors];
    	    5,  DATA_ENTRY_ERROR, [too_few_data_bytes_recieved];
            32, MODE_ERROR,       [busy];
    	}
}

/****************************************************************/


MENU device_state
{
    LABEL "|en|Device state|de|Gerte Zustand";
    STYLE GROUP;
    ITEMS
    {
        NAMUR_state (DISPLAY_VALUE)
#if 0
        ,
        COLUMNBREAK,
        Out_spec
#endif
    }
}

MENU diagnostic_root_menu
{
    LABEL MENU_DIAGNOSE;
    STYLE WINDOW;
    ITEMS
    {
        device_state,
	DeviceDiagApplication,
	DeviceDiagSensor,
	DeviceDiagElectronics
    }
}

MENU simulation
{
    LABEL MENU_SIMULATE;
    STYLE WINDOW;
    ITEMS
    {
        transmitter_loop_test
    }
}

MENU extra
{
    LABEL "Extra";
    STYLE MENU;
    ITEMS
    {

#ifdef _PDM_
        reset_to_factory,
        set_write_protection
#else
        reset_to_factory
#endif
    }
}

VARIABLE write_protection_state
{
    LABEL "|en|Write protection|de|Schreibschutz";
    HELP  "";
    CLASS DEVICE;
    HANDLING READ & WRITE;
    DEFAULT_VALUE 1;
    TYPE ENUMERATED
    {
        { 1, "|en|Write protected|de|Schreibschutz" },
        { 0, "|en|Not protected|de|Kein Schreibschutz"   }
    }
}

VARIABLE software_write_protection_state
{
    LABEL "|en|Software write protection|de|Software Schreibschutz";
    HELP  "";
    CLASS DEVICE;
    HANDLING READ & WRITE;
    DEFAULT_VALUE 1;
    TYPE ENUMERATED
    {
        { 0, "|en|Write protected|de|Schreibschutz" },
        { 1, "|en|Not protected|de|Kein Schreibschutz"   }
    }
}

VARIABLE password
{
    LABEL "|en|password|de|Pawort";
    CLASS DEVICE;
    HANDLING WRITE;
    TYPE PACKED_ASCII (4);
}

COMMAND set_software_write_protection_state
{
    NUMBER 180;
    OPERATION WRITE;
    TRANSACTION
    {
        REQUEST
        {
            write_protection_state,
            password
        }
        REPLY
        {
            response_code, device_status,
            software_write_protection_state,
            password
        }
    }
    RESPONSE_CODES
    {
        0,  SUCCESS,          [no_command_specific_errors];
        2,  DATA_ENTRY_ERROR, [invalid_selection];
        3,  DATA_ENTRY_ERROR, [passed_parameter_too_large];
        4,  DATA_ENTRY_ERROR, [passed_parameter_too_small];
        5,  DATA_ENTRY_ERROR, [too_few_data_bytes_recieved];
        7,  MODE_ERROR,       [in_write_protect_mode];
        16, MODE_ERROR,       [access_restricted];
        32, MODE_ERROR,       [busy];
    }
}

COMMAND reset_to_factory_data
{
    NUMBER 189;
    OPERATION COMMAND;
    TRANSACTION
    {
        REQUEST
        {
        }
        REPLY
        {
            response_code, device_status
        }
    }
    RESPONSE_CODES
    {
        0,  SUCCESS,          [no_command_specific_errors];
        6,  MISC_ERROR,       [xmtr_specific_command_error];
        7,  MODE_ERROR,       [in_write_protect_mode];
        16, MODE_ERROR,       [access_restricted];
    }
}

METHOD reset_to_factory
{
    LABEL "|en|Reset to factory|de|Reset zu Fabrikszustand";
    HELP "|en|Perform a Reset to Factory Data.\n\n|de|Ein Reset auf die Fabrikseinstellungen durchfhren";
        
    CLASS DEVICE;
    DEFINITION
    {
        char status[STATUS_SIZE];

        char disp_string[60];
        int slen;

        slen = 60;

        ACKNOWLEDGE("|en|Really reset to factory settings?|de|Wirklich Reset auf die Fabrikseinstellungen");

        send(189, status);
        if (status[STATUS_RESPONSE_CODE])
        {
            display_response_status(189, status[STATUS_RESPONSE_CODE]);
            get_dictionary_string(abort_last_error, disp_string, slen);
            DELAY(2, disp_string);
            process_abort();
        }
    }
}

#ifdef _PDM_
METHOD set_write_protection
{
    LABEL "|en|Set write protection|de|Setze Schreibschutz";
    CLASS DEVICE;
    DEFINITION
    {
        char status[STATUS_SIZE];

        char disp_string[60];
        int slen;

        int result;

        slen = 60;

        IGNORE_ALL_DEVICE_STATUS();
        XMTR_IGNORE_ALL_DEVICE_STATUS();

        result = SELECT_FROM_LIST("|en|Activate write protection?|de|Aktiviere Schreibschutz?", "|en|Yes;No|de|Ja;Nein" );

        if (result == 0)
        { // he answered "yes"
            software_write_protection_state = 1;
            password = "0000";
            send(180, status);
            if (status[STATUS_RESPONSE_CODE])
            {
                display_response_status(180,status[STATUS_RESPONSE_CODE]);
                get_dictionary_string(abort_last_error, disp_string, slen);
                DELAY(2, disp_string);
                process_abort();
            }
        }
        else
        {

            software_write_protection_state = 0;
            GET_DEV_VAR_VALUE("|en|Enter password|de|Pawort Eingabe", password);
            send(180, status);
            if (status[STATUS_RESPONSE_CODE])
            {
                display_response_status(180,status[STATUS_RESPONSE_CODE]);
                get_dictionary_string(abort_last_error, disp_string, slen);
                DELAY(2, disp_string);
                process_abort();
            }
        }
   }
}

MENU write_protection
{
    LABEL "|en|Write protection|de|Schreibschutz";
    STYLE MENU;
    ITEMS
    {
        set_write_protection
    }
}

REFRESH write_protection_relation
{
    software_write_protection_state:
        write_protection_state,
        write_protect
}
#endif

VARIABLE pv_percentRange
{
    LABEL [pv_percent_range];
    HELP [percent_of_range_help];
    CLASS RANGE & DYNAMIC;
    HANDLING READ;
    CONSTANT_UNIT [Percent];
    TYPE FLOAT
    {
        DISPLAY_FORMAT "3.2f";
    }
}


VARIABLE pv_upperRange_value
{
    LABEL "|en|To (20mA)|de|Zu (20mA)";
    HELP [upper_range_value_help];
    CLASS RANGE;
    TYPE FLOAT
    {
        DISPLAY_FORMAT "3.2f";
        EDIT_FORMAT "3.2f";
    }
}

VARIABLE pv_lowerRange_value
{
    LABEL "|en|Range (4mA)|de|Bereich (4mA)";
    HELP [lower_range_value_help];
    CLASS RANGE;
    TYPE FLOAT
    {
        DISPLAY_FORMAT "3.2f";
        EDIT_FORMAT "3.2f";
    }
}

VARIABLE pv_range_units
{
    HELP [digital_units_help];
    LABEL "|en|Range unit|de|Bereichseinheit";
    TYPE ENUMERATED
    {
        ALL_TTX_UNITS
    }
    CLASS INPUT & SERVICE;
    HANDLING READ & WRITE;
}

VARIABLE pv_loopCurrent
{
    LABEL "|en|Analog Value|de|Analog Wert";
    HELP [loop_current_help];
    CLASS ANALOG_CHANNEL & DYNAMIC;
    HANDLING READ;
    CONSTANT_UNIT [mA];
    TYPE FLOAT
    {
        DISPLAY_FORMAT "3.2f";
        EDIT_FORMAT "3.2f";
    }
}

COMMAND write_pv_range_values
{
    NUMBER 35;
    OPERATION WRITE;
    TRANSACTION
    {
        REQUEST
        {
            pv_range_units, pv_upperRange_value, pv_lowerRange_value
        }
        REPLY
        {
            response_code, device_status,
            pv_range_units, pv_upperRange_value, pv_lowerRange_value
        }
    }
    RESPONSE_CODES
    {
        0,  SUCCESS,            [no_command_specific_errors];
        2,  DATA_ENTRY_ERROR,   [invalid_selection];
        5,  DATA_ENTRY_ERROR,   [too_few_data_bytes_recieved];
        6,  MISC_ERROR,         [xmtr_specific_command_error];
        7,  MODE_ERROR,         [in_write_protect_mode];
        9,  DATA_ENTRY_ERROR,   [lower_range_value_too_high];
        10, DATA_ENTRY_ERROR,   [lower_range_value_too_low];
        11, DATA_ENTRY_ERROR,   [upper_range_value_too_high];
        12, DATA_ENTRY_ERROR,   [upper_range_value_too_low];
        13, DATA_ENTRY_ERROR,   [URV_and_LRV_out_of_limits];
        14, DATA_ENTRY_WARNING, [span_too_small];
        16, MODE_ERROR,         [access_restricted];
        32, MODE_ERROR,         [busy];
    }
}
